<?php
/**
 * CarrierBox front controller service
 *
 * @author    Serge <serge@stigmi.eu>
 * @copyright 2015 Stigmi
 * @license   http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */

require_once(dirname(__FILE__).'/FrontControllerService.php');
require_once(_PS_MODULE_DIR_.'bpostshm/classes/Service.php');

class ServicePointService extends FrontControllerService
{
	const GMAPS_API_KEY = 'AIzaSyAa4S8Br_5of6Jb_Gjv1WLldkobgExB2KY';

	public function processContent()
	{
		$shipping_method = Tools::getValue('shipping_method');
		if ($delivery_date = Tools::getValue('dd'))
			$delivery_date = date('d-m-Y', strtotime($delivery_date));

		$service = new Service($this->context);
		$module = $service->module;
		$cart_bpost = PsCartBpost::getByPsCartID((int)$this->context->cart->id);

		//a//
		if (Tools::getValue('get_nearest_service_points'))
		{
			$search_params = array('zone' => '',);
			$postcode = Tools::getValue('postcode');
			$city = Tools::getValue('city');
			if ($postcode)
				$search_params['zone'] .= (int)$postcode.($city ? ' ' : '');
			if ($city)
				$search_params['zone'] .= (string)$city;
			if ($delivery_date)
				$search_params['dd'] = (string)$delivery_date;

			$service_points = (BpostShm::SHM_PPOINT == $shipping_method) ?
				$service->getNearestServicePoint($search_params) :
				$service->getNearestServicePoint($search_params, $shipping_method);
			$this->jsonEncode($service_points);
		}
		elseif (Tools::getValue('get_service_point_hours'))
		{
			$service_point_id = (int)Tools::getValue('service_point_id');
			$sp_type = (int)Tools::getValue('sp_type');
			$service_point_hours = $service->getServicePointHours($service_point_id, $sp_type);
			$this->jsonEncode($service_point_hours);
		}
		elseif (Tools::getValue('set_service_point'))
		{
			$service_point_id = (int)Tools::getValue('service_point_id');
			$sp_type = (int)Tools::getValue('sp_type');
			$this->jsonEncode($cart_bpost->setServicePoint($service_point_id, $sp_type));
		}
		elseif (Tools::getValue('post_upl_unregister'))
		{
			$upl_info = (string)Tools::getValue('post_upl_info');
			$stored = $upl_info === (string)$cart_bpost->upl_info;
			if (!$stored)
			{
				$cart_bpost->upl_info = $upl_info;
				$stored = $cart_bpost->save();
			}

			$this->jsonEncode($stored);
		}

		//p//
		self::$smarty->assign('version', (Service::isPrestashop16plus() ? 1.6 : (Service::isPrestashop15plus() ? 1.5 : 1.4)), true);
		self::$smarty->assign('module_dir', _MODULE_DIR_.$module->name.'/', true);
		self::$smarty->assign('shipping_method', $shipping_method, true);
		switch ($shipping_method)
		{
			case BpostShm::SHM_PPOINT:
				$named_fields = $service->getNearestValidServicePoint(3, $delivery_date);
				foreach ($named_fields as $name => $field)
					self::$smarty->assign($name, $field, true);

				$get_nearest_link_params = array(
						'mode' 							=> 'service-point',
						'ajax'							=> true,
						'get_nearest_service_points' 	=> true,
						'shipping_method'				=> $shipping_method,
						'token'							=> Tools::getToken($module->name),
				);
				if ($delivery_date) $get_nearest_link_params['dd'] = $delivery_date;
				self::$smarty->assign('url_get_nearest_service_points', $service->getBpostLink($get_nearest_link_params));
				self::$smarty->assign('url_get_service_point_hours', $service->getBpostLink(
					array(
						'mode' 						=> 'service-point',
						'ajax'						=> true,
						'get_service_point_hours' 	=> true,
						'shipping_method'			=> $shipping_method,
						'token'						=> Tools::getToken($module->name),
				)));
				self::$smarty->assign('url_set_service_point', $service->getBpostLink(
					array(
						'mode' 				=> 'service-point',
						'ajax'				=> true,
						'set_service_point' => true,
						'shipping_method'	=> $shipping_method,
						'token'				=> Tools::getToken($module->name),
				)));

				// $this->addJqueryPlugin('scrollTo');
				$this->setTemplate('map-servicepoint.tpl');
				break;

			case BpostShm::SHM_PLOCKER:
				$step = (int)Tools::getValue('step', 1);
				switch ($step)
				{
					default:
					case 1:
						self::$smarty->assign('step', 1, true);

						$delivery_address = new Address($this->context->cart->id_address_delivery, $this->context->language->id);
						// UPL
						$upl_info = Tools::jsonDecode($cart_bpost->upl_info, true);
						if (!isset($upl_info))
							$upl_info = array(
								'eml' => $this->context->customer->email,
								'mob' => !empty($delivery_address->phone_mobile) ? $delivery_address->phone_mobile : '',
								'rmz' => false,
								);

						$iso_code = $this->context->language->iso_code;
						$upl_info['lng'] = in_array($iso_code, array('fr', 'nl')) ? $iso_code : 'en';
						self::$smarty->assign('upl_info', $upl_info, true);
						//
						self::$smarty->assign('url_post_upl_unregister', $service->getBpostLink(
							array(
								'mode' 					=> 'service-point',
								'ajax'					=> true,
								'post_upl_unregister' 	=> true,
								'shipping_method'		=> $shipping_method,
								'token'					=> Tools::getToken($module->name),
						)));

						$get_point_list_link_params = array(
								'mode' 				=> 'service-point',
								'content_only'		=> true,
								'shipping_method'	=> $shipping_method,
								'step'				=> 2,
								'token'				=> Tools::getToken($module->name),
						);
						if ($delivery_date) $get_point_list_link_params['dd'] = $delivery_date;
						self::$smarty->assign('url_get_point_list', $service->getBpostLink($get_point_list_link_params));

						$this->setTemplate('form-upl.tpl');
						break;

					case 2:
						$named_fields = $service->getNearestValidServicePoint($shipping_method, $delivery_date);
						foreach ($named_fields as $name => $field)
							self::$smarty->assign($name, $field, true);

						$get_nearest_link_params = array(
								'mode' 							=> 'service-point',
								'ajax'							=> true,
								'get_nearest_service_points' 	=> true,
								'shipping_method'				=> $shipping_method,
								'token'							=> Tools::getToken($module->name),
						);
						if ($delivery_date) $get_nearest_link_params['dd'] = $delivery_date;
						self::$smarty->assign('url_get_nearest_service_points', $service->getBpostLink($get_nearest_link_params));
						self::$smarty->assign('url_get_service_point_hours', $service->getBpostLink(
							array(
								'mode' 						=> 'service-point',
								'ajax'						=> true,
								'get_service_point_hours' 	=> true,
								'shipping_method'			=> $shipping_method,
								'token'						=> Tools::getToken($module->name),
						)));
						self::$smarty->assign('url_set_service_point', $service->getBpostLink(
							array(
								'mode' 				=> 'service-point',
								'ajax'				=> true,
								'set_service_point' => true,
								'shipping_method'	=> $shipping_method,
								'token'				=> Tools::getToken($module->name),
						)));

						// $this->addJqueryPlugin('scrollTo');
						$this->setTemplate('map-servicepoint.tpl');
						break;
				}
				break;
		}
	}

	public static function setMedia(FrontController $ctlr)
	{
		parent::setMedia($ctlr);

		$base_uri = __PS_BASE_URI__.'modules/bpostshm/views/';
		// $base_uri = __PS_BASE_URI__.'modules/'.$ctlr->module->name.'/views/';
		$ctlr->addCSS($base_uri.'css/servicepoint.css');
		$ctlr->addCSS($base_uri.'css/jquery.qtip.min.css');
		//
		// $ctlr->addJS('//cdn.jsdelivr.net/jquery.scrollto/2.1.0/jquery.scrollTo.min.js');
		$ctlr->addJS($base_uri.'js/eon.jquery.base.min.js');
		$ctlr->addJS($base_uri.'js/eon.jquery.servicepointer.min.js');
		$ctlr->addJS($base_uri.'js/jquery.qtip.min.js');
		/*
		$ctlr->addJS('https://maps.googleapis.com/maps/api/js?v=3.16&key='.static::GMAPS_API_KEY.'&sensor=false&language='
			// .$ctlr->context->language->iso_code);
		*/
		$ctlr->addJS('https://maps.googleapis.com/maps/api/js?v=3&key='.static::GMAPS_API_KEY.'&language=en');
	}
}