<?php
/**
 * Generic front controller (for Prestashop 1.4)
 *
 * @author    Serge <serge@stigmi.eu>
 * @version   1.30.0
 * @copyright Copyright (c), Eontech.net All rights reserved.
 * @license   BSD License
 */

const _CONFIG_FILE_ = '../../../../config/config.inc.php';
const _CONFIG_FILE_DEV_ = '../../../../sites/bpost/sps14/config/config.inc.php';
const _MODULE_NAME_ = 'bpostshm';

if (file_exists(_CONFIG_FILE_))
	require_once(_CONFIG_FILE_);
elseif (file_exists(_CONFIG_FILE_DEV_))
	require_once(_CONFIG_FILE_DEV_);
else
	die('Cannot locate config');

class ClientBpost extends FrontController
{
	public $module;
	public $ssl = true;

	private $tpl;
	private $css_files = array();
	private $js_files = array();

	public function __construct()
	{
		$this->controller_type = 'modulefront';

		$this->css_files[] = _THEME_CSS_DIR_.'global.css';
		$this->js_files[] = '//ajax.googleapis.com/ajax/libs/jquery/1.7.2/jquery.min.js';

		parent::__construct();
	}

	public function run()
	{
		$this->init();
		$this->preProcess();
		$this->displayHeader();
		$this->process();
		$this->displayContent();
	}

	public function init()
	{
		parent::init();

		// initialising the module anywhere before here...
		$this->module = Module::getInstanceByName(_MODULE_NAME_);
		if (!$this->module->active)
			Tools::redirect('index');

		// ...severely affects the returned context ?!
		if (!isset($this->context))
		{
			$this->context = Context::getContext();
			$this->context->controller = $this;
		}
	}

	public function displayHeader()
	{
		if (!Tools::getValue('ajax', false))
		{
			foreach ($this->css_files as $css)
				echo '<link href="'.$css.'" type="text/css" rel="stylesheet" />';
			foreach ($this->js_files as $script)
				echo '<script src="'.$script.'" type="text/javascript"></script>';

		}
	}

	public function process()
	{
		$svc = $this->getServiceName();
		$token = Tools::getValue('token');
		$gen_token = (bool)Tools::getValue('admin') ? Tools::getAdminToken($this->module->name) : Tools::getToken($this->module->name);
		if (!$svc || $token !== $gen_token)
			Tools::redirect('index');

		parent::process();

		$svc = new $svc($this->context);
		$svc->processContent();
	}

	public function displayContent()
	{
		parent::displayContent();

		self::$smarty->display($this->tpl);
	}

	public function setTemplate($tpl)
	{
		$this->tpl = _PS_MODULE_DIR_.'bpostshm/views/templates/front/'.$tpl;
	}

	public function addCSS($css_uri)
	{
		$this->css_files[] = $css_uri;
	}

	public function addJS($js_uri)
	{
		$this->js_files[] = $js_uri;
	}

	public function setMedia()
	{
		parent::setMedia();

		if ($svc = $this->getServiceName())
			$svc::setMedia($this);
	}

	public function getServiceName()
	{
		if ($service_name = Tools::getValue('mode', false))
		{
			$service_name = str_replace('-', '', preg_replace_callback('/(\b[a-z])\-*/', function($matches) {
				return Tools::strtoupper($matches[0]);
			}, $service_name)).'Service';
			$service_file = dirname(__FILE__).'/'.$service_name.'.php';
			if (file_exists($service_file))
			{
				require_once($service_file);
				if (!class_exists($service_name, false))
					return false;
			}
		}

		return $service_name;
	}
}

$controller = new ClientBpost();
$controller->run();
